import sass

autogen_warning = "/* Autogenerated by gutta - DO NOT EDIT. */\n\n"

def build_style(sources:list[str]) -> str:
    return autogen_warning+sass.compile(string = "\n\n".join(sources),output_style='compressed')